#ifndef IAOP_H
#define IAOP_H

/* CIAO - C I-AM-ON API VER 1.0
 *
 * This file defines the C API to IAOP (I Am On Protocol).
 * IAOP is used by agents to cummunicate with the R-U-ON site.
 * Check r-u-on.com for more information.
 *
 * Copyright R-U-ON, 2006
 * www.r-u-on.com 
 */

/*
 * If you are compiling this file you might want to define OS
 * to match your operating system.
 * For most unix compilers you need to add -DOS=\"MyOS\" to the cmd
 * line. For Visual Studio you have to add OS=\"MyOS\" in the 
 * preprocessor defines.
 * For windows platform to compile, make sure WIN32 is defined.
 */
#ifndef OS
#define OS "ccode"
#endif


#define IAOP_MAX_LEN 512

/* The following two functions should be defined in the calling process (i.e.
 * YOU should implement them).
 * They are called by the API to log applicative error messages or system
 * call error messages respectively.
 * When iao_logsysmsg is called it is an indication that you should check
 * the last system error messages (errno / GetLastError / ...)
 * msg: Contains the applicative message describing what IAMON API was trying 
 *      to do at the point of failure.
 */
void iao_logmsg(char *msg);
void iao_logsysmsg(char *msg);

/*
 * This structure contains the agent information and should be
 * passed for each call to the API.
 */
struct iao_agentinfo_struct {
   char type[IAOP_MAX_LEN];    /* The type of the agent (for instance ServerOn) */
   char version[IAOP_MAX_LEN]; /* The agent's version */
   char id[IAOP_MAX_LEN];      /* The agent's unique id. When calling getAgentId, should  */
};		                /* be equal to account id */
typedef struct iao_agentinfo_struct iao_agentinfo;

/* This functions returns the agent's IP from the last successful 
 * connection to the R-U-ON server			
 */
char *iao_getmylastip();

/* iao_init should be called once, before any other call to the library.
 * 
 * return value: The function returns 0 in case of success and !0 incase of 
 *               failure. The function will write to the log the failure reason.
 */
int iao_init();

/* iao_register should be called once per installation. The result is
 * a unique id to be used by the agent. This id should be persisted to
 * be reused by the agent instance.
 *
 * agent:           A structure identifying the agent type and the account id.
 * result_agent_id: A buffer that will hold the new agent id. The buffer has to
 *                  be at least of size IAOP_MAX_LEN.
 * skaa:	    suppress-keep-alive-alarms. When !0, the server will create
 *                  the agent with the Keep-Alive-Alarms option turned off.
 *                  This is useful for agents monitor end user devices such
 *                  as laptops and desktops.
 *
 * return value: The function returns 0 in case of success and !0 incase of 
 *               failure. The function will write to the log the failure reason.
 */
int iao_register(iao_agentinfo *agent, char *result_agent_id, int skaa);


/* iao_keepalive should be called to report the agent is on and doing well.
 * The result of this function is a directive to be carried by the
 * calling process.
 *
 * agent: A structure identifying the agent type and the agent id.
 * verb:  A buffer that will hold the verb of the directive to carry. Buffer has
 *        to be at least IAOP_MAX_LEN bytes. possible values are (between the 
 *        parenthesis):
 *        "sleep":     The agent should wait for X seconds before making contact
 *                     again. If the agent does not make contact within X
 *                     seconds it will be pronounced dead. If the agent makes
 *                     contact before X seconds it may be pronounced delinquent.
 *                     The agent is allowed to make contact before X seconds have
 *                     elapsed if it is reporting changes in the active alarms state.
 *        "configure": The agent should call iao_fetchconfig. X represents the number
 *                     of seconds the agent should wait after fetching configuration
 *                     and before calling the next keepalive (In this context X is 
 *                     similar to the sleep directive).
 *        "upgrade":   The agents needs to perform an upgrade. Only agents that
 *                     have been dealt by r-u-on can receive this directive.
 *        "uninstall": The agent needs to uninstall. This is a result of a
 *                     user's request. If the agent does not have the abilty to
 *                     uninstall, it should at least die. Further API calls will
 *                     fail.
 * x:     A buffer that will hold the value for X. X has different meaning
 *        depending on the value of verb. See the different verbs for more info.
 *
 * return value: The function returns 0 in case of success and !0 incase the data is the same or if
 *               the call to iao_alarms failed. The function will write to the log the failure reason.
 */
int iao_keepalive(iao_agentinfo *agent, char *verb, char *x);

/*
 * The iao_alarm structure is used to notify the server of alarms (up, down, event)
 * Alarm up: Tells the server that the alarm should be raised. If the alarm
 *           already exists, the alarm values will change according to the 
 *           new structure.
 * Alarm down: Tells the server that an alarm should be cleared.
 * Event: Tells the server of an event that occured. The server will notify
 *        the user and keep the event in the history file but will not show
 *        the alarm in the active alarms screen.
 * The unique id of the alarm is a combination of the resource and the id.
 * The resource identifies the monitored resource the alarm refers to and
 * the id identifies the alarm within the resource (incase the same resource
 * can have multiple alarms at the same time). The server maintains the alarms
 * per agent so it is not neccessary to add the agent id into the unique identifier
 */
struct iao_alarm_struct {
   char type;              /* 'A': Alarm 'C': Clear 'E': Event */
   char resource[256];     /* The monitored resource the alarm refers to */
   char id[256];           /* The alarm unique identifier (with resource) */
   char severity;          /* 'C': Critical 'M': Major 'm': Minor (ignored for clear) */
   char description[1024]; /* Alarm description. You can leave this empty for alarm clear */
			   /* If you populate for alarm clear, the value will be used for */
			   /* User notifications. */
};
typedef struct iao_alarm_struct iao_alarm;


/* iao_alarms is used to report changes in the alarm list (alarm, clear, event).
 * The function get's a list of alarms, and the mode in which they should be
 * processed. There are two supported modes:
 * incremental: The agent reports changes in the alarm list state.
 * snapshot:    The agent reports all the alarms that are currently active.
 *              Any active alarm that is not reported in the snapshot is cleared.
 * The result of this function is a directive to be carried by the
 * calling process.
 *
 * agent:  A structure identifying the agent type and the agent id.
 * alarms: An array of alarms.
 * count:  The length of the alarms array.
 * inc:    Indicates which mode to use. Pass 1 for incremental and 0 for snapshot
 * verb:   A buffer that will hold the verb of the directive to carry. Buffer has
 *         to be at least IAOP_MAX_LEN bytes. See iao_keepalive for possible
 *         values.
 * x:      A buffer that will hold the value for X. X has different meaning
 *         depending on the value of verb. See the different verbs for more info.
 *
 * return value: The function returns 0 in case of success and !0 in case of
 *               failure. The function will write to the log the failure reason.
 */
int iao_alarms(iao_agentinfo *agent, iao_alarm alarms[], int count, int inc,
               char *verb, char *x);

/* iao_alarms_ifchange is similar to iao_alarms but will only report alarms
 * in case a change has been since the last time the function has been called.
 * To compare the function checks alarm IDs and resources of Alarms (clear records
 * are ignroed, the descripiton and severity fields are ignored).
 * The function always uses the snapshot mode.
 * In addition if an hour or more has passed since the last time a change occured and there 
 * are alarms they will be sent anyway. This serves to update alarm descriptions from time
 * to time.
 * The function is can be used to pass some of the agent logic to the API. Specifically,
 * you can skip any validation of state change and always report alarms at any frequency.
 * the alarms will only be sent if a real change has occured.
 * agent:  A structure identifying the agent type and the agent id.
 * alarms: An array of alarms.
 * count:  The length of the alarms array.
 * inc:    Indicates which mode to use. Pass 1 for incremental and 0 for snapshot
 * verb:   A buffer that will hold the verb of the directive to carry. Buffer has
 *         to be at least IAOP_MAX_LEN bytes. See iao_keepalive for possible
 *         values.
 * x:      A buffer that will hold the value for X. X has different meaning
 *         depending on the value of verb. See the different verbs for more info.
 *
 * return value: The function returns 0 in case of success and !0 in case of
 *               failure. The function will write to the log the failure reason.
 */
int iao_alarms_ifchange(iao_agentinfo *agent, iao_alarm alarms[], int count, 
               char *verb, char *x);


struct iao_confparam_struct {
	char *name;
	char *type;
	char *defvalue;
};
typedef struct iao_confparam_struct iao_confparam;

/*
 * iao_fetchconfig should be called by agents that allow users to configure
 * them at the beginning of their execution and also as a response to a 
 * config directive.
 * The agent specifies which parameters the user can manipulate per agent
 * and also optionally per managed resource (in case the agent manages a list of 
 * other resources such as hosts, porcesses etc).
 * These are refered to as "agent level" and "resource level" parameters.
 * For each parameter the agent can specify a default value and a type.
 *
 * agent:  A structure identifying the agent type and the agent id.
 * agentconf: An array of arrays, terminated by an array of NULLs. Can be NULL to
 *            indicate the agent does not expect any agent level parameters.
 *            The array has three elements: name, type and default value.
 *            Currently type can be "String" or "Integer".
 * resources: This argument has the same structure as the agent argument and
 *            indicates if the agent expects a managed resource list and what
 *            arguments are expected per managed resource.
 *            The first argument in the list is the managed resource id.
 *            Pass NULL to indicate the agent does not expect a resource list.
 *
 * return value: The function returns 0 in case of success and !0 incase of
 *               failure. The function will write to the log the failure reason.
 *
 */
int iao_fetchconfig(iao_agentinfo *agent, 
                    iao_confparam *agentconf, iao_confparam *resources);

/*
 * After a successful call to iao_fetchconfig, use this function to get the
 * value of a configuration parameter (agent level).
 *
 * name: The name of the parameter.
 *
 * return value: A pointer to the value. If an integer is expected, use atoi.
 *               If there is no such parameter, you will get a NULL.
 *               The pointer will become invalid after the next call to
 *               iao_fetchconfig.
 */
char *iao_configvalue(char *name);



/*
 * After a sucessful call to iao_fetchconfig, use this function to get
 * a list of managed services (hosts, processes, etc.).
 *
 * return value: A pointer to a NULL delimited list of managed resrouces. After
 *               the last resource there is an extra NULL to indicate the end of
 *               the list.
 *               If there are no resrouces, you will get a NULL.
 *               The pointer will become invalid after the next call to 
 *               iao_fetchconfig so you are encouraged to maintain a reference to
 *               this list.
 */
char *iao_configresourcelist();


/*
 * After a successful call to iao_fetchconfig, use this function to get the 
 * value of a configuration parameter (managed resource level).
 * 
 * resource: The name of the resource as returned by iao_configresources.
 * name:      The name of the parameter.
 *
 * return value: A pointer to the value. If an integer is expected, use atoi.
 *               If there is no such parameter, you will get a NULL.
 *               The pointer will become invalid after the next call to
 *               iao_fetchconfig so you are ancouraged to maintain a reference to it.
 */
char *iao_configresourcevalue(char *resource, char *name);


/* iao_getbinary should be called as a result of directive to upgrade. The
 * function returns a buffer containing the new binary file.
 * Note that the abilty for automatic upgrades is reserved for agents that have
 * been delivered by the r-u-on site.
 *
 * agent:     A structure identifying the agent type and the agent id.
 * buff:      A pointer to a (void*) that will end up holding the binary file
 *            (new version). The function dynamically allocates the memory that
 *            holds the buffer and the user should call free() to return this
 *            memory to the heap after it is used.
 * len:       A pointer to an interger that will hold the length 
 *
 * return value: The function returns 0 in case of success and !0 incase of
 *               failure. The function will write to the log the failure reason.
 */
int iao_getbinary(iao_agentinfo *agent, void **buff, int *size);




/* iao_setparams is used by agents that need to set parameters that are usually 
 * configured using the R-U-ON API. Most agents do not need to use this method
 * unless some local information or UI are used to set a parameter.
 * The parameters that can be set by the agent are: config parameters, resources,
 * alias, group, notification severity, keep alive alarm supression.
 *
 * agent:      A structure identifying the agent type and the account id.
 * namevalues: Pass an array of iao_param representing name/value pairs to set the values you want.
 *             Terminate the array with a NULL in the name member.
 *             The possible names and values are as follows:
 *
 *             IAO_AGENTPARAM_ALIAS 
 *             The value is the new alias for the agent
 *
 *             IAO_AGENTPARAM_GROUP
 *             The new group for the agent
 *
 *             IAO_AGENTPARAM_NOTIF
 *             Notification severity. This must be one of the following:
 *             "Minor", "Major", "Critical", "none"
 *
 *             IAO_AGENTPARAM_SKAA
 *             Suppress Keepalive Alarms. To activate the value must be "true". Any other value will
 *             deactivate the suppression.
 *
 *             IAO_AGENTPARAM_RESOURCES
 *             The resources data chunk. The convention for the value is exactly like the agent resource
 *             string on the web interface: A list of lines seperated by \r\n where the values inside
 *             the line are seperated by commas.
 *
 *             Any other name is regarded as a configuration paramters
 *            
 *             Example:
 *             iao_param namevalues[] = { 
 *                                        {IAO_AGENTPARAM_ALIAS, "MYALIAS"},
 *                                        {IAO_AGENTPARAM_GROUP, "MYGROUP"},
 *                                        {NULL, NULL}
 *                                      };
 *
 * return value: The function returns 0 in case of success and !0 incase of
 *               failure. The function will write the failure reason to the log.
 */

#define IAO_AGENTPARAM_ALIAS "alias"
#define IAO_AGENTPARAM_GROUP "group"
#define IAO_AGENTPARAM_NOTIF "notif"
#define IAO_AGENTPARAM_SKAA "skaa"
#define IAO_AGENTPARAM_RESOURCES "resources"

typedef struct 
{ 
	char *name;
	char *value;
} iao_param;


int iao_setparams(iao_agentinfo *agent, iao_param namevalues[]);



/* iao_getembeddedvariable fetches configuration parameters that were embedded 
 * in the binary file. This is done during the download process from the r-u-on
 * site. As a result this function is only usefull for shared agents (i.e.
 * agents that have been uploaded to r-u-on. This function should not be used by
 * user agents.
 *
 * name:   The name of the variable to fetch.
 * result: A pointer to buffer of IAOP_MAX_LEN size, that will receive the
 *         desired value. The value returned will be a null terminated string.
 *
 * return value: The function returns 0 in case of success and !0 incase of
 *               failure. A failure will occur if the variable is not found.
 */
int iao_getembeddedvariable(char *name, char *result);



#endif
